#The jet propagation in a medium. Modeled by Ehud Nakar, implemented by Ore Gottlieb.
#The function HeadLoc receives the medium density profile in rho (g/cm^3) on the z axis (z), and the jet luminosity in Lj(erg/s) and opening angle in theta0(rad).
#The function HeadLoc returns three vectors. The jet head location (zh) and velocity/c (bh) at time (t). 
import numpy as np
import math
from scipy import interpolate
from scipy.optimize import root
def find_Lt( Lt,z,rho,Lj,theta0):
    Luncal=(Lj/theta0**4/rho/z**2/3e10**3*16/math.pi)**0.4*(1/(1+1/np.sqrt(Lt)))**0.8
    Lcol=theta0**(-4/3)*16**(2/3)
    if Luncal<1:
        Ns=0.35
    elif Luncal<Lcol:
        Ns=0.35*Luncal**(0.456/math.log10(Lcol))
    else:
        Ns=1
    Lt1=Ns**2*Luncal
    Lt2=Lj/theta0**2/rho/z**2/3e10**3/math.pi*(1/(1+Lt**-0.5))**2
    x=Lt-max(Lt2,Lt1)
    return x
def Ltalp2(z,rho,Lj,theta0):
#    Lt=fsolve(lambda x : find_Lt(x,z,rho,Lj,theta0),1e-6)
    def func(x):
        return find_Lt(x,z,rho,Lj,theta0)
    Lt = root(func, 0.5).x[0]
    return Lt
def HeadLoc(z,rho,theta0,Lj):
    bh,Lt,t,gh,Omega,Pc,bc,zh,rc,Rho,M,V,varRho,zeta,ksi,eps = [],[],[],[],[],[],[],[],[],[],[],[],[],[],[],[]
    Lt.append(Ltalp2(z[0],rho[0],Lj,theta0))
    bh.append(1/(1+Lt[0]**-0.5))
    t.append(z[0]/3e10/bh[0])
    gh.append(math.sqrt(1/(1-bh[0]**2)))
    Omega.append(3)
    Pc.append((Lj*rho[0]/3/math.pi/3e10)**0.5*Lt[0]**-0.25/t[0]*Omega[0]**0.5)
    bc.append((Pc[0]/rho[0]/3e10**2/3)**0.5)
    zh.append(z[0])
    rc.append(bc[0]*t[0]*3e10)
    Rho.append(rho[0])
    M.append(4*math.pi*Rho[0]*zh[0]**3)
    V.append(4/3*math.pi*zh[0]**3)
    varRho.append(0)
    ksi.append(0)
    zeta.append(0)
    eps.append(0)
    int_zeta0=bh[0]*t[0]
    int_ksi0=bc[0]*t[0]
    int_eps0=1/gh[0]**2*t[0]
    k=0
    while (zh[k]<z[-1]):
        k=k+1
        t.append(t[k-1]*(1+0.01))
        La=(Lj/theta0**4/t[k]**2/Rho[k-1]/3e10**5)**0.4*(16*Omega[k-1]/3/math.pi)**0.4
        Lcol=theta0**(-4/3)*(16*Omega[k-1]/3)**(2/3)
        if La<1:
            Ns=0.35
        elif La<Lcol:
            Ns=0.35*La**(0.46/math.log10(Lcol))
        else:
            Ns=1
        Lt1=Ns**2*La
        Lt2=Lj/math.pi/theta0**2/t[k]**2/Rho[k-1]/3e10**5
        Lt.append(max(Lt1,Lt2))
        bh.append(1/(1+Lt[k]**-0.5))
        gh.append(math.sqrt(1/(1-bh[k]**2)))
        zh.append(zh[k-1]+bh[k]*(t[k]-t[k-1])*3e10)
        rc.append(rc[k-1]+bc[k-1]*(t[k]-t[k-1])*3e10)
        Rho.append(math.exp(interpolate.interp1d(np.log(z),np.log(rho),kind='linear',fill_value='extrapolate')(math.log(zh[k]))))
        Pc.append(1/3/math.pi*Lj*(t[k]-zh[k]/3e10)/zh[k]/rc[k]**2)
        M.append(M[k-1]+4*math.pi*zh[k]**2*(zh[k]-zh[k-1])*Rho[k])
        V.append(V[k-1]+4*math.pi*zh[k]**2*(zh[k]-zh[k-1]))
        varRho.append(M[k]/V[k]/Rho[k])
        bc.append((Pc[k]/varRho[k]/Rho[k]/3e10**2)**.5)
        zeta.append((int_zeta0+np.trapz(bh[0:k+1],t[0:k+1]))/t[k]/bh[k])
        ksi.append((int_ksi0+np.trapz(bc[0:k+1],t[0:k+1]))/t[k]/bc[k])
        eps.append((int_eps0+np.trapz(np.power(gh[0:k+1],-2),t[0:k+1]))/t[k]/gh[k]**(-2))
        Omega.append(eps[k]*varRho[k]/zeta[k]/ksi[k]**2)
    import matplotlib.pyplot as plt
    plt.plot(zh,bh)
    plt.title('\beta(zh)')
    plt.ylabel('beta_h')
    plt.xlabel('z_h')
    return t,zh,bh